import transformers as t
from PIL import Image
import os
import pandas as pd
from mb_utils.src import logging
from pp_weight_estimation.utils.slack_connect import send_message_to_slack
from pp_weight_estimation.core.get_weight import get_seg, get_final_img, get_histogram, get_val, split_filename, get_final_mask_filter
from pp_weight_estimation.core.s3_io import  download_image,get_response
from pp_weight_estimation.core.gpt_support import get_count,get_gpt_result,get_gpt_result2,multi_ref_prompt
from typing import List,Dict,Union
import yaml
import boto3
from datetime import datetime
from tqdm import tqdm
import numpy as np
import cv2

logger = logging.logger 
#model_checkpoint = '/Users/test/test1/mit-segformer-s' 
#model = t.TFSegformerForSemanticSegmentation.from_pretrained(model_checkpoint)

__all__ = ["load_color_values", "process_pipeline"]

def load_config(yaml_path: str) -> dict:
    """
    Function to load configurations from a YAML file
    Args:
        yaml_path (str): Path to the YAML file
    Returns:
        dict: Dictionary containing configurations
    """
    with open(yaml_path, 'r') as file:
        config = yaml.safe_load(file)
    return config

def load_color_values_old(csv_path : str, logger: logging.Logger = None) -> Dict:
    """
    Function to load color values from a CSV file
    Args:
        csv_path (str): Path to the CSV file
        logger (Logger): Logger object
    Returns:
        dict: Dictionary containing color values
    """
    if logger:
        logger.info("Loading color values from CSV")
    color_dict = {}
    df = pd.read_csv(csv_path)
    for _, row in df.iterrows():
        taxcode = row['taxonomy_code']
        site = row['site_id']
        color = row['colors']
        if taxcode and site and color:
            composed_key = f"{site}_{taxcode}"
            color_list = eval(color)
            color_dict[composed_key] = color_list
    return color_dict

def load_color_values(csv_path : str, logger: logging.Logger = None) -> Dict:
    """
    Function to load color values from a CSV file
    Args:
        csv_path (str): Path to the CSV file
        logger (Logger): Logger object
    Returns:
        dict: Dictionary containing color values
    """
    if logger:
        logger.info("Loading color values from CSV")
    color_dict = {}
    df = pd.read_csv(csv_path)
    for _, row in df.iterrows():
        item_code = row['item_code']
        model_version = row['model_version']
        #site = row['site_id']
        color = row['colors']
        # if taxcode and site and color:
        #     composed_key = f"{site}_{taxcode}"
        #     color_list = eval(color)
        #     color_dict[composed_key] = color_list
        if item_code and model_version and pd.isna(color) is False:
            composed_key = f"{item_code}_{model_version}"
            color_list = eval(color)
            color_dict[composed_key] = color_list
        else:
            composed_key = f"{item_code}_{model_version}"
            color_dict[composed_key] = None
    return color_dict

def process_pipeline(config_path: str, logger: logging.Logger = None, **kwargs) -> Union[pd.DataFrame, List]:
    """
    Function to process the pipeline of Production Planning.
    
    This function automates the process of downloading images from an S3 bucket, applying segmentation and masking,
    calculating histograms, and saving the results to an output CSV file. The function uses configurations provided
    in a YAML file for flexibility.

    Args:
        config_path (str): Path to the YAML configuration file.
        logger (logging.Logger): Logger object for logging messages (optional).
        **kwargs: Additional keyword arguments (optional).
        
    Returns:
        tuple: A tuple containing:
            - str: The path to the output CSV file.
            - list: A list of results for each processed image.
    """
    config = load_config(config_path)

    input_csv_path = config['data']['input_csv_path']
    #reference_file = config['data']['gt_csv_path']
    val_color_csv_path = config['data']['val_color_csv_path']
    # logger = config['data']['logger']
    # if logger:
    #     logger = logging.logger

    replace_images = config['data'].get('replace_images', False)
    path_to_save_images = config['data']['path_to_save_images']
    if path_to_save_images[-1] != '/':
        path_to_save_images += '/'
    annotation_data_res = config['data'].get('annotation_data_res', False)

    save_plots = config['results'].get('save_plots', False)
    final_mask_vals = config['results'].get('final_mask_vals', 50)
    mask_model_val = config['results'].get('model_val', 0.08)
    mask_val = config['results'].get('mask_val', 0.3)
    new_bg_removal = config['results'].get('background_removal', True)
    equalizer_items = config['results'].get('equalizer_items')
    slack_post = config['results'].get('slack_post', False)
    slack_webhook = config['results'].get('slack_webhook', None)

    model_path = config['model']['model_path']
    model_name = config['model']['model_name']
    model_version = config['model']['model_version']

    bucket = config['aws_data']['bucket_name']
    profile = config['aws_data']['profile']

    gpt_response = config['gpt_res'].get('gpt_response', False)
    #gpt_token = config['gpt_res'].get('gpt_token', None)
    #gpt_model = config['gpt_res'].get('gpt_model', None)
    gpt_prompt = config['gpt_res'].get('gpt_prompt', None)
    #gpt_file_path = config['gpt_res'].get('gpt_file_path', None)
    gpt_api_key = config['gpt_res']['gpt_api_key']

    output_file_name = config['data'].get('output_file_name', None)
    todays_date = "{:%Y_%m_%d}".format(datetime.now())


    model = t.TFSegformerForSemanticSegmentation.from_pretrained(model_path)  

    session = boto3.Session()
    client = session.client(profile)


    color_dict = load_color_values(val_color_csv_path)
    # groundtruth_df = pd.read_csv(gt_csv_path)
    input_df = pd.read_csv(input_csv_path)
    if logger:
        logger.info("Length of input dataframe: {}".format(len(input_df)))
    input_df = input_df[input_df['s3_image_path'].notna()]
    input_df = input_df[input_df['s3_reference_image_path'].notna()]
    input_df.reset_index(drop=True, inplace=True)
    if logger:
        logger.info("Length of input dataframe after removing null values: {}".format(len(input_df)))
        
    input_df['ground_truth_g']=input_df['ground_truth_kg']*1000
    input_df['rework_g']=input_df['rework_kg']*1000
    input_df['mask'] = ''
    input_df['final_image'] = ''
    input_df['pixel_count'] = 0
    input_df['histogram'] = ''
    input_df['pred_w2'] = 0.0
    input_df['error'] = 0.0
    input_df['success'] = False
    model_final_name = model_name + model_version
    input_df['model_version']= model_final_name
    if annotation_data_res:
        input_df['annotation_result'] = 0.0
        input_df['annotation_error'] = 0.0
        input_df['annotation_success'] = False
    if gpt_response:
        input_df['gpt_result'] = 0.0
        input_df['gpt_error'] = 0.0
        input_df['gpt_success'] = False
        input_df['gpt_count_res'] = False
        input_df['gpt_count_value'] = 0.0

    #use_input_groundtruth = 'input_groundtruth' in input_df.columns

    input_view_image_bb=[]
    input_view_image_mask=[]
    ref_view_image_bb=[]
    ref_view_image_mask=[]

    entries = []
    for _, row in input_df.iterrows():
        s3_image_path = row['s3_image_path']
        image_dict = split_filename(row['s3_image_path'])
        local_image_folder = path_to_save_images + image_dict['site_id'] + '/' + image_dict['food_item'] + '/'
        os.makedirs(local_image_folder, exist_ok=True)
        local_image_path = path_to_save_images + image_dict['site_id'] + '/' + image_dict['food_item'] + '/' + image_dict['image'] + '.jpeg'
        if os.path.exists(local_image_path) and replace_images is False:
            # if logger:
            #     logger.info(f"Image {local_image_path} already exists. Skipping download.")   
            pass
        else:
            img_temp = download_image(bucket, row['s3_image_path'], client)
            img_temp.save(local_image_path)
        site_id = image_dict['site_id']
        #taxonomy_code = row['taxonomy_code']
        food_item = image_dict['food_item']
        #input_groundtruth = row['input_groundtruth'] if use_input_groundtruth else 0

        food_item_label = row['food_item_label']
        s3_reference_image_path = row['s3_reference_image_path']
        if s3_reference_image_path is None:
            if logger:
                logger.error(f"No reference image found for image {local_image_path}. Skipping image.")
        else:
            s3_reference_weight = row['ground_truth_g']
            rework_g = row['rework_kg']*1000
            if local_image_path and site_id and food_item:
                #composed_key = f"{site_id}_{taxonomy_code}"
                entries.append((site_id, local_image_path, food_item,local_image_folder,image_dict['image'],s3_image_path,
                                s3_reference_image_path,s3_reference_weight,rework_g,model_final_name,food_item_label))
    
    if not entries:
        if logger:
            logger.error("No valid entries found in the CSV file")
        raise ValueError("No valid entries found in the CSV file")
    
    #results = []
    for index, (site_id, local_image_path, food_item,local_image_folder,image_dict_name,s3_image_path,reference_image,reference_image_gt,rework_g,model_final_name,
                food_item_label) in enumerate(tqdm(entries, desc="Processing images",leave=False)):
        try:
            key = f"{food_item}_{model_final_name}"
            if key not in color_dict:
                if logger:
                    logger.error(f"No color found for key {food_item}. Skipping image {local_image_path}.")
                input_df.at[index, 'success'] = False
            colors = color_dict[key]
        
            if food_item in equalizer_items:
                equalizer = True
            else:
                equalizer = False
            masked_img = get_seg(local_image_path, model, mask_val=mask_model_val, resize=True, new_bg_removal=new_bg_removal, equalize=equalizer)

            final_mask = get_final_mask_filter( masked_img, colors=colors, val=final_mask_vals, logger=logger)

            # if save_plots:
            #     mask_image = Image.fromarray(final_mask.astype(np.uint8))
            #     mask_image_path = local_image_folder + f"{food_item}_mask_{final_mask_vals}.png"
            #     mask_image.save(mask_image_path)
            #     input_df.at[index, 'mask'] = mask_image_path
            # else:
            #     input_df.at[index, 'mask'] = ''

            new_final_img = get_final_img(local_image_path, final_mask,mask_val)  ## saving final mask with the image
            if save_plots:
                #final_image_path = local_image_folder + image_dict_name + "_" +todays_date  +"_final_mask.jpeg"
                final_image_path = local_image_folder + image_dict_name + "_final_mask.jpeg"
                temp_img= Image.fromarray(new_final_img)
                temp_img.save(final_image_path)
                input_df.at[index, 'mask'] = final_image_path
            else:
                input_df.at[index, 'mask'] = ''

            new_histogram, _ = get_histogram(new_final_img)

            pixel_count = new_histogram[-1]
            pixel_count = pixel_count.astype(int)
            input_df.at[index, 'pixel_count'] = pixel_count

            # if save_plots:
            #     plt.figure()
            #     plt.plot(new_histogram)
            #     plt.savefig(local_image_folder + f"{image_dict_name}_{todays_date}_histogram.jpeg")
            #     plt.close()

            #reference_row = groundtruth_df[groundtruth_df['taxonomy_code'] == taxonomy_code]
            # if not reference_row.empty:
            #     reference_pixel_count = reference_row.iloc[0]['reference_pixel_count']
            #     groundtruth_weight = reference_row.iloc[0]['groundtruth']

            #     weight2 = input_groundtruth if input_groundtruth else 0
            #     pred_w2, error = get_val(reference_pixel_count, pixel_count, groundtruth_weight, weight2)
            # else:
            #     if logger:
            #         logger.error(f"No groundtruth data found for taxonomy_code {taxonomy_code}.")
        
            #reference_pixel_count, reference_image_weight =get_reference_data(reference_image,model_version,reference_file)
            
            ## get refrence image pixel data
            if os.path.exists(reference_image) is False:
                ref_image = download_image(bucket, reference_image, client)
                ref_dict = split_filename(reference_image)
                ref_image_path = path_to_save_images + ref_dict['site_id'] + '/' + ref_dict['food_item'] + '/' + ref_dict['image'] + '.jpeg'
                ref_image.save(ref_image_path)
            masked_img_ref = get_seg(ref_image_path, model, mask_val=mask_model_val, resize=True, new_bg_removal=new_bg_removal, equalize=equalizer)
            final_mask_ref = get_final_mask_filter(masked_img_ref, colors=colors, val=final_mask_vals,show=False, logger=logger)
            new_final_img_ref = get_final_img(ref_image_path, final_mask_ref,mask_val)  ## saving final mask with the image
            if save_plots:
                final_image_path_ref = local_image_folder + ref_dict['image'] +"_final_mask.jpeg"
                if os.path.exists(final_image_path_ref):
                    os.remove(final_image_path_ref)
                temp_img= Image.fromarray(new_final_img_ref)
                temp_img.save(final_image_path_ref)
            new_histogram_ref, _ = get_histogram(new_final_img_ref)
            pixel_count_ref = new_histogram_ref[-1]
            pixel_count_ref = pixel_count_ref.astype(int)

            pred_w2, error = get_val(pixel_count_ref, pixel_count, reference_image_gt, rework_g)

            input_df.at[index, 'pred_w2'] = pred_w2
            input_df.at[index, 'error'] = error
            input_df.at[index, 'success'] = True if pred_w2 > 0.00 else False

        except Exception as e:
            if logger:
                logger.error(f"Error processing image {local_image_path}: {e}")
            input_df.at[index, 'success'] = False

        if annotation_data_res:
            main_image_dict = {}
            main_image_dict['bb_path'] ='labelbox/' + s3_image_path[:-13] + '/bounding_box.json'
            main_image_dict['mask_path'] = 'labelbox/' + s3_image_path[:-13] + '/mask.jpg'
            ref_image_dict = {}
            ref_image_dict['bb_path'] ='labelbox/' + reference_image[:-13] + '/bounding_box.json'
            ref_image_dict['mask_path'] = 'labelbox/' + reference_image[:-13] + '/mask.jpg'
            # print(main_image_dict)
            # print(ref_image_dict)
            labelbox_bucket='pp-image-weight-estimation-euwest1-uat'

            var_img=False
            var_ref_img=False
            try:
                if main_image_dict['bb_path'] is not None:
                    client.get_object(Bucket=labelbox_bucket, Key=main_image_dict['bb_path'])
                    client.get_object(Bucket=labelbox_bucket, Key=main_image_dict['mask_path'])
                    var_img=True
            except:
                if logger:
                    logger.error(f"Error downloading annotation data for image {s3_image_path}")
            
            try:
                if ref_image_dict['bb_path'] is not None:
                    client.get_object(Bucket=labelbox_bucket, Key=ref_image_dict['bb_path'])
                    client.get_object(Bucket=labelbox_bucket, Key=ref_image_dict['mask_path'])
                    var_ref_img=True
            except:
                if logger:
                    logger.error(f"Error downloading annotation data for ref image {ref_image_dict['bb_path']}")

            if var_img and var_ref_img:
                try:
                    main_image_s3_bb = get_response(client,labelbox_bucket,main_image_dict['bb_path'])
                    main_image_bb = eval(main_image_s3_bb['Body'].read().decode('utf-8'))
                    main_image_mask = download_image(labelbox_bucket,main_image_dict['mask_path'],client)
                    main_image_mask.save(local_image_folder + image_dict_name + "_annotation_mask.jpeg")

                    ref_image_s3_bb = get_response(client,labelbox_bucket,ref_image_dict['bb_path'])
                    ref_image_bb = eval(ref_image_s3_bb['Body'].read().decode('utf-8'))
                    ref_image_mask = download_image(labelbox_bucket,ref_image_dict['mask_path'],client)

                    x1_bb,y1_bb,x2_bb,y2_bb  = int(main_image_bb['top']),int(main_image_bb['left']),int(main_image_bb['height']),int(main_image_bb['width'])
                    x1_ref_bb,y1_ref_bb,x2_ref_bb,y2_ref_bb  = int(ref_image_bb['top']),int(ref_image_bb['left']),int(ref_image_bb['height']),int(ref_image_bb['width'])

                    image_data = np.array(Image.open(local_image_path))
                    mask_data = np.array(main_image_mask)
                    bbox_image = image_data[x1_bb:(x2_bb+x1_bb),y1_bb:(y2_bb+y1_bb)]
                    bbox_mask = mask_data[x1_bb:(x2_bb+x1_bb),y1_bb:(y2_bb+y1_bb)]
                    bbox_image2 = cv2.resize(bbox_image, (512,512))
                    bbox_mask2 = cv2.resize(bbox_mask, (512,512))

                    ref_image_data = np.array(Image.open(ref_image_path))
                    ref_mask_data = np.array(ref_image_mask)
                    ref_bbox_image = ref_image_data[x1_ref_bb:(x2_ref_bb+x1_ref_bb),y1_ref_bb:(y2_ref_bb+y1_ref_bb)]
                    ref_bbox_mask = ref_mask_data[x1_ref_bb:(x2_ref_bb+x1_ref_bb),y1_ref_bb:(y2_ref_bb+y1_ref_bb)]
                    ref_bbox_image2 = cv2.resize(ref_bbox_image, (512,512))
                    ref_bbox_mask2 = cv2.resize(ref_bbox_mask, (512,512))
        
                    hist_mask,_ = get_histogram(bbox_mask2)
                    hist_ref_mask,_ = get_histogram(ref_bbox_mask2)
                    # print(hist_mask[-1])
                    # print(hist_ref_mask[-1])
                    img_weight,img_error = get_val(hist_ref_mask[-1],hist_mask[-1],reference_image_gt,rework_g)

                    input_df.at[index, 'annotaion_result']  = round(img_weight,2)
                    input_df.at[index, 'annotation_error'] =  round(img_error,2)
                    input_df.at[index, 'annotation_success'] = True

                    input_view_image_bb.append(bbox_image2)
                    input_view_image_mask.append(bbox_mask2)
                    ref_view_image_bb.append(ref_bbox_image2)
                    ref_view_image_mask.append(ref_bbox_mask2)

                except Exception as e:
                    if logger:
                        logger.error(f"Error processing annotation data for image {s3_image_path}")
                    input_df.at[index, 'annotation_success'] = False
                    input_df.at[index, 'annotation_error'] = 0.0
                    input_df.at[index, 'annotation_result'] = 0.0
            else:
                input_df.at[index, 'annotation_success'] = False
                input_df.at[index, 'annotation_error'] = 0.0
                input_df.at[index, 'annotation_result'] = 0.0

        if gpt_response:
            gpt_prompt = f"Based on the reference image of weight {food_item_label} : {reference_image_gt} grams, what is the weight of the 2nd image? Please respond only in the format 'amount' without grams unit."
            try:
                gpt_result = get_gpt_result(image_path = local_image_path, ref_image_path= ref_image_path, api_key = gpt_api_key, prompt = gpt_prompt)
                gpt_final_result = get_count(gpt_result) 
            except:
                gpt_final_result = False

            if gpt_final_result!=False:
                try:
                    gpt_error = (abs(rework_g-gpt_final_result)/rework_g)*100 if rework_g !=0 else 0.0
                    input_df.at[index, 'gpt_success'] = True 
                    print(f'gpt_error for {food_item_label}: {gpt_error}')
                except:
                    gpt_error = 0.0
                    input_df.at[index, 'gpt_success'] = False
            else:
                gpt_error = 0.0
                input_df.at[index, 'gpt_success'] = False
            input_df.at[index, 'gpt_error'] = gpt_error
            print(f'gpt_result for {food_item_label}: {gpt_final_result}')
            input_df.at[index, 'gpt_result'] = gpt_final_result
        
            gpt_prompt2 = f"If the {food_item_label} item can be counted. Please respond only the count in the format 'amount' and if it cannot be counted respond 'False'."
            try:
                gpt_result2 = get_gpt_result2(image_path = local_image_path, api_key = gpt_api_key, prompt = gpt_prompt2)
                gpt_final_result2 = get_count(gpt_result2) 
            except:
                gpt_final_result2 = False

            if gpt_final_result2!=False:
                gpt_count_value = gpt_final_result2
                input_df.at[index, 'gpt_count_res'] = True
                # try:
                #     input_df.at[index, 'gpt_count_res'] = True 
                #     #print(f'gpt_count for {food_item_label}: {gpt_final_result2}')
                # except Exception as e:
                #     gpt_count_value = 0.0
                #     input_df.at[index, 'gpt_count_res'] = False
            else:
                gpt_count_value = 0.0
                input_df.at[index, 'gpt_count_res'] = False
            input_df.at[index, 'gpt_count_value'] = gpt_count_value
            print(f'gpt_count for {food_item_label}: {gpt_count_value}')
    
    output_csv_dir = path_to_save_images + 'output_csv/'
    os.makedirs(output_csv_dir, exist_ok=True)
    if output_file_name is not None:
        output_csv_path = os.path.join(output_csv_dir, output_file_name)
    else:
        output_csv_path = os.path.join(output_csv_dir, f"output_{todays_date}.csv")
    input_df.to_csv(output_csv_path, index=False)

    if logger:
        logger.info(f"Processing complete. Output saved to {output_csv_path}")

    try:
        temp_pd = input_df[['site_id','food_item_label','rework_g','pred_w2','error','success']]
        if annotation_data_res:
            temp_pd['annotation_result'] = input_df['annotation_result']
            temp_pd['annotation_error'] = input_df['annotation_error']
            temp_pd['annotation_success'] = input_df['annotation_success']

        if gpt_response:
            temp_pd['gpt_result'] = input_df['gpt_result']
            temp_pd['gpt_error'] = input_df['gpt_error']
            temp_pd['gpt_count_value'] = input_df['gpt_count_value']
            temp_pd['gpt_count_res'] = input_df['gpt_count_res']
    except Exception as e:
        if logger:
            logger.error(f"Error sending message to slack: {e}")
    
    if slack_post:
        send_message_to_slack(temp_pd, is_df=True,url=slack_webhook,tabulate_type='fancy_grid')

    return input_df,temp_pd,input_view_image_bb,input_view_image_mask,ref_view_image_bb,ref_view_image_mask
    #return output_csv_path, results


def chatgpt_only_pipeline(config_path: str, prompt: str = None,use_annotate_data: str = False,gpt_reference:str = False,bb_img:str = False,logger: logging.Logger = None, **kwargs) -> Union[pd.DataFrame, List]:
    """
    Function to process the pipeline of Production Planning.
    
    This function automates the process of downloading images from an S3 bucket, applying segmentation and masking,
    calculating histograms, and saving the results to an output CSV file. The function uses configurations provided
    in a YAML file for flexibility.

    Args:
        config_path (str): Path to the YAML configuration file.
        prompt (str): Prompt for ChatGPT model.
        use_annotate_data (bool): Use annotation data for processing.
        gpt_reference (bool): Use reference image for GPT.
        bb_img (bool): Use bounding box image for processing.
        logger (logging.Logger): Logger object for logging messages (optional).
        **kwargs: Additional keyword arguments (optional).
        
    Returns:
        tuple: A tuple containing:
            - str: The path to the output CSV file.
            - list: A list of results for each processed image.
    """
    config = load_config(config_path)

    input_csv_path = config['data']['input_csv_path']

    replace_images = config['data'].get('replace_images', False)
    path_to_save_images = config['data']['path_to_save_images']
    if path_to_save_images[-1] != '/':
        path_to_save_images += '/'

    bucket = config['aws_data']['bucket_name']
    profile = config['aws_data']['profile']

    gpt_api_key = config['gpt_res']['gpt_api_key']

    output_file_name = config['data'].get('output_file_name', None)
    todays_date = "{:%Y_%m_%d}".format(datetime.now())

    session = boto3.Session()
    client = session.client(profile)

    input_df = pd.read_csv(input_csv_path)
    if logger:
        logger.info("Length of input dataframe: {}".format(len(input_df)))
    input_df = input_df[input_df['s3_image_path'].notna()]
    input_df = input_df[input_df['s3_reference_image_path'].notna()]
    input_df.reset_index(drop=True, inplace=True)
    if logger:
        logger.info("Length of input dataframe after removing null values: {}".format(len(input_df)))
        

    #input_df['ground_truth_g']=input_df['ground_truth_kg']*1000
    input_df['rework_g']=input_df['rework_kg']*1000

    input_df['gpt_result'] = 0.0
    input_df['gpt_error'] = 0.0
    input_df['gpt_success'] = False
    input_df['gpt_count_res'] = False
    input_df['gpt_count_value'] = 0.0

    input_view_image_bb=[]
    ref_view_image_bb=[]

    entries = []
    for _, row in input_df.iterrows():
        s3_image_path = row['s3_image_path']
        image_dict = split_filename(row['s3_image_path'])
        local_image_folder = path_to_save_images + image_dict['site_id'] + '/' + image_dict['food_item'] + '/'
        os.makedirs(local_image_folder, exist_ok=True)
        local_image_path = path_to_save_images + image_dict['site_id'] + '/' + image_dict['food_item'] + '/' + image_dict['image'] + '.jpeg'
        if os.path.exists(local_image_path) and replace_images is False:
            # if logger:
            #     logger.info(f"Image {local_image_path} already exists. Skipping download.")   
            pass
        else:
            img_temp = download_image(bucket, row['s3_image_path'], client)
            img_temp.save(local_image_path)
        site_id = image_dict['site_id']
        #taxonomy_code = row['taxonomy_code']
        food_item = image_dict['food_item']
        #input_groundtruth = row['input_groundtruth'] if use_input_groundtruth else 0

        food_item_label = row['food_item_label']
        # try:
            # s3_reference_image_path = eval(row['s3_reference_image_path'])
        # except:
        #     s3_reference_image_path = row['s3_reference_image_path']
        temp_s3_reference_image_path = row['s3_reference_image_path']
        if isinstance(temp_s3_reference_image_path,list):
            ref_list = eval(temp_s3_reference_image_path)
            print(f'ref_list: {ref_list}')
            new_ref_list = []
            for i in ref_list:
                ref_dict = split_filename(i)
                local_image_folder_ref = path_to_save_images + ref_dict['site_id'] + '/' + ref_dict['food_item'] + '/'
                os.makedirs(local_image_folder_ref, exist_ok=True)
                local_image_path_ref = path_to_save_images + ref_dict['site_id'] + '/' + ref_dict['food_item'] + '/' + ref_dict['image'] + '.jpeg'
                if os.path.exists(local_image_path_ref) and replace_images is False:
                    pass
                else:
                    img_temp = download_image(bucket, i, client)
                    img_temp.save(local_image_path_ref)
                new_ref_list.append(local_image_path_ref)
            # s3_reference_image_path = eval(row['s3_reference_image_path']) ## making every reference image path as list
        else:
            s3_reference_image_path = [row['s3_reference_image_path']]
        s3_reference_weight = eval(row['ground_truth_kg'])
        if isinstance(s3_reference_weight,list):
            s3_reference_weight = [x*1000 for x in s3_reference_weight]
            rework_g = row['rework_kg']*1000
        else:
            s3_reference_weight = s3_reference_weight*1000
            rework_g = row['rework_kg']*1000

        if s3_reference_image_path is None:
            if logger:
                logger.error(f"No reference image found for image {local_image_path}. Skipping image.")
        else:
            print(f's3_path_check: {s3_reference_image_path}')

            if len(new_ref_list) > 1:
                print(f'Processing multi reference image for {local_image_path}')
                prompt_multi_ref = "Which image is the most similar to the first image? Just respond with number {'image': value}"   
                multi_ref_res = multi_ref_prompt(local_image_path, new_ref_list, prompt_multi_ref, gpt_api_key)
                if multi_ref_res:
                    multi_ref_res = eval(multi_ref_res)
                    print(multi_ref_res)
                    multi_ref_res_image = multi_ref_res['image']
                    s3_reference_weight = s3_reference_weight[multi_ref_res_image-1]
                    s3_reference_image_path = new_ref_list[multi_ref_res_image-1]
                else:
                    if logger:
                        logger.error(f"Error processing list input {local_image_path}")
            else:
                s3_reference_image_path = s3_reference_image_path[0]

            print(f's3_path : {s3_reference_image_path}')
            print(f'reference_weight : {s3_reference_weight}')
            if local_image_path and site_id and food_item:
                #composed_key = f"{site_id}_{taxonomy_code}"
                entries.append((site_id, local_image_path, food_item,local_image_folder,image_dict['image'],s3_image_path,
                                s3_reference_image_path,s3_reference_weight,rework_g,food_item_label))

    if not entries:
        if logger:
            logger.error("No valid entries found in the CSV file")
        raise ValueError("No valid entries found in the CSV file")
    
    #results = []
    for index, (site_id, local_image_path, food_item,local_image_folder,image_dict_name,s3_image_path,reference_image,reference_image_gt,rework_g,
                food_item_label) in enumerate(tqdm(entries, desc="Processing images",leave=False)):
        def prompt_valid(prompt,food_item_label,reference_image=None,reference_image_gt=None):
            if reference_image is not None:
                gpt_prompt = prompt.format(food_item_label=food_item_label,reference_image=reference_image,reference_image_gt=reference_image_gt)
            else:
                gpt_prompt = prompt.format(food_item_label=food_item_label)
            return gpt_prompt
        print(f"Processing image {local_image_path}")
        print(f"Processing reference image {reference_image}")
        print(f"Processing food item {food_item_label}")
        if gpt_reference:
            gpt_prompt = prompt_valid(prompt,food_item_label,reference_image=reference_image,reference_image_gt=reference_image_gt)
        else:
            gpt_prompt = prompt_valid(prompt,food_item_label)

        if os.path.exists(reference_image) is False:
            ref_image = download_image(bucket, reference_image, client)
            ref_dict = split_filename(reference_image)
            ref_image_path = path_to_save_images + ref_dict['site_id'] + '/' + ref_dict['food_item'] + '/' + ref_dict['image'] + '.jpeg'
            ref_image.save(ref_image_path)

        if use_annotate_data:
            main_image_dict = {}
            main_image_dict['bb_path'] ='labelbox/' + s3_image_path[:-13] + '/bounding_box.json'
            ref_image_dict = {}
            ref_image_dict['bb_path'] ='labelbox/' + reference_image[:-13] + '/bounding_box.json'

            labelbox_bucket='pp-image-weight-estimation-euwest1-uat'

            var_img=False
            var_ref_img=False
            try:
                if main_image_dict['bb_path'] is not None:
                    client.get_object(Bucket=labelbox_bucket, Key=main_image_dict['bb_path'])
                    var_img=True
            except:
                if logger:
                    logger.error(f"Error downloading annotation data for image {s3_image_path}")
            
            try:
                if ref_image_dict['bb_path'] is not None:
                    client.get_object(Bucket=labelbox_bucket, Key=ref_image_dict['bb_path'])
                    var_ref_img=True
            except:
                if logger:
                    logger.error(f"Error downloading annotation data for ref image {ref_image_dict['bb_path']}")

            if var_img and var_ref_img:
                try:
                    main_image_s3_bb = get_response(client,labelbox_bucket,main_image_dict['bb_path'])
                    main_image_bb = eval(main_image_s3_bb['Body'].read().decode('utf-8'))

                    ref_image_s3_bb = get_response(client,labelbox_bucket,ref_image_dict['bb_path'])
                    ref_image_bb = eval(ref_image_s3_bb['Body'].read().decode('utf-8'))

                    x1_bb,y1_bb,x2_bb,y2_bb  = int(main_image_bb['top']),int(main_image_bb['left']),int(main_image_bb['height']),int(main_image_bb['width'])
                    x1_ref_bb,y1_ref_bb,x2_ref_bb,y2_ref_bb  = int(ref_image_bb['top']),int(ref_image_bb['left']),int(ref_image_bb['height']),int(ref_image_bb['width'])

                    image_data = np.array(Image.open(local_image_path))
                    bbox_image = image_data[x1_bb:(x2_bb+x1_bb),y1_bb:(y2_bb+y1_bb)]
                    bbox_image2 = cv2.resize(bbox_image, (512,512))

                    ref_image_data = np.array(Image.open(ref_image_path))
                    ref_bbox_image = ref_image_data[x1_ref_bb:(x2_ref_bb+x1_ref_bb),y1_ref_bb:(y2_ref_bb+y1_ref_bb)]
                    ref_bbox_image2 = cv2.resize(ref_bbox_image, (512,512))

                    input_view_image_bb.append(bbox_image2)
                    ref_view_image_bb.append(ref_bbox_image2)

                except Exception as e:
                    if logger:
                        logger.error(f"Error processing annotation data for image {s3_image_path}")
            if bb_img and var_img and var_ref_img:
                temp_ref_img = Image.fromarray(ref_view_image_bb[-1])
                temp_ref_img.save('./'+"ref_bb.jpeg")
                temp_orig_img = Image.fromarray(input_view_image_bb[-1])
                temp_orig_img.save('./'+"orig_bb.jpeg")
                local_image_path = './'+"orig_bb.jpeg"
                ref_image_path = './'+"ref_bb.jpeg"
            else:
                if logger:
                    if var_img is False:
                        logger.error(f"Error downloading annotation data for image {s3_image_path}")
                    if var_ref_img is False:
                        logger.error(f"Error downloading annotation data for ref image {ref_image_dict['bb_path']}")
                    logger.info('Using original image data for processing')

        if gpt_reference:
            #gpt_prompt = f"Based on the reference image of weight {food_item_label} : {reference_image_gt} grams, what is the weight of the 2nd image? Please respond only in the format 'amount' without grams unit."
            # gpt_prompt = prompt.format(food_item_label,reference_image_gt)

            gpt_result = get_gpt_result(image_path = local_image_path, ref_image_path= ref_image_path, api_key = gpt_api_key, prompt = gpt_prompt)
            gpt_final_result = get_count(gpt_result) 

            if gpt_final_result!=False:
                try:
                    gpt_error = (abs(rework_g-gpt_final_result)/rework_g)*100 if rework_g !=0 else 0.0
                    input_df.at[index, 'gpt_success'] = True 
                    print(f'gpt_error for {food_item_label}: {gpt_error}')
                except:
                    gpt_error = 0.0
                    input_df.at[index, 'gpt_success'] = False
            else:
                gpt_error = 0.0
                input_df.at[index, 'gpt_success'] = False
            input_df.at[index, 'gpt_error'] = gpt_error
            print(f'gpt_result for {food_item_label}: {gpt_final_result}')
            input_df.at[index, 'gpt_result'] = gpt_final_result
        
        else:
            # gpt_prompt = f"If the {food_item_label} item can be counted. Please respond only the count in the format 'amount' and if it cannot be counted respond 'False'."
            # gpt_prompt = prompt.format(food_item_label)
            gpt_result = get_gpt_result2(image_path = local_image_path, api_key = gpt_api_key, prompt = gpt_prompt)
            gpt_final_result = get_count(gpt_result) 

            if gpt_final_result!=False:
                gpt_count_value = gpt_final_result
                input_df.at[index, 'gpt_count_res'] = True
            else:
                gpt_count_value = 0.0
                input_df.at[index, 'gpt_count_res'] = False
            input_df.at[index, 'gpt_count_value'] = gpt_count_value
            print(f'gpt_count for {food_item_label}: {gpt_count_value}')
    
    output_csv_dir = path_to_save_images + 'output_csv/'
    os.makedirs(output_csv_dir, exist_ok=True)
    if output_file_name is not None:
        output_csv_path = os.path.join(output_csv_dir, output_file_name)
    else:
        output_csv_path = os.path.join(output_csv_dir, f"output_{todays_date}.csv")
    input_df.to_csv(output_csv_path, index=False)

    if logger:
        logger.info(f"Processing complete. Output saved to {output_csv_path}")

    return input_df,input_view_image_bb,ref_view_image_bb


# We will use the OpenAI API to get the result of the image

import base64
import requests

__all__ = ['get_gpt_result','encode_image','get_count','get_gpt_result2','multi_ref_prompt']

prompt = "Count the number of {item} in the image and just return the number"
#gpt_prompt = f"Based on the reference image of weight {food_item_label} : {reference_image_gt}, what is the weight of the 2nd image? Please respond only in the format 'result : amount'."


# OpenAI API Key
api_key = None

# Function to encode the image
def encode_image(image_path):
    with open(image_path, "rb") as image_file:
        return base64.b64encode(image_file.read()).decode('utf-8')


def get_gpt_result(image_path ,ref_image_path, api_key,prompt=prompt,detail="high"):
    # Getting the base64 string
    base64_image = encode_image(image_path)
    ref_base64_image = encode_image(ref_image_path)

    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {api_key}"}

    payload = {
    "model": "gpt-4o",
    "messages": [
        {
        "role": "user",
        "content": [
            {
            "type": "text",
            "text": prompt
            },
            {
            "type": "image_url",
            "image_url": {
                "url": f"data:image/jpeg;base64,{ref_base64_image}",
                 "detail": detail
            }
            },
            {            
            "type": "image_url",
            "image_url": {
                "url": f"data:image/jpeg;base64,{base64_image}",
                 "detail": detail
            }
            },
        ]
        }
    ],
    "max_tokens": 300}

    response = requests.post("https://api.openai.com/v1/chat/completions", headers=headers, json=payload)

    print(response.json())
    return response


def get_gpt_result2(image_path , api_key,prompt=prompt,detail="high"):
    # Getting the base64 string
    base64_image = encode_image(image_path)
    headers = {
        "Content-Type": "application/json",
        "Authorization": f"Bearer {api_key}"}

    payload = {
    "model": "gpt-4o",
    "messages": [
        {
        "role": "user",
        "content": [
            {
            "type": "text",
            "text": prompt
            },
            {
            "type": "image_url",
            "image_url": {
                "url": f"data:image/jpeg;base64,{base64_image}",
                "detail" : detail
            }
            },
        ]
        }
    ],
    "max_tokens": 300}

    response = requests.post("https://api.openai.com/v1/chat/completions", headers=headers, json=payload)

    print(response.json())
    return response

def get_count(response):
    try :
        result = float(response.json()['choices'][0]['message']['content'])
    except Exception as e:
        print(e)
        result = False
    return result

def multi_ref_prompt(image_path, ref_image_list, api_key,prompt=prompt,detail="high"):
    """
    Multiple reference images in list format
    Args:
        image_path : str : Path to the image
        ref_image_list : list : List of reference images
        api_key : str : OpenAI API Key
        prompt : str : Prompt to be used
        detail : str : Detail of the image
    Returns:
        response : dict : Response from the API
    """

    base64_image = encode_image(image_path)
    ref_base64_images = [encode_image(ref_image) for ref_image in ref_image_list]

    data = {
        "image": base64_image,
        "references": ref_base64_images,
        "prompt": prompt,
        "detail": detail,
        "model" : "gpt-4o",
    }
    
    headers = {
        "Authorization": f"Bearer {api_key}",
        "Content-Type": "application/json"
    }
    
    response = requests.post("https://api.openai.com/v1/chat/completions", json=data, headers=headers)
    
    if response.status_code == 200:
        return response.json()['choices'][0]['message']['content']
    else:
        return False


import pytest
import pandas as pd
import os
from pp_weight_estimation.core.function_test import process_pipeline

class MockLogger:
    def info(self, message):
        print(f"INFO: {message}")
    def error(self, message):
        print(f"ERROR: {message}")

logger = MockLogger()

def test_process_pipeline():
    input_csv = "tests/sample_input.csv"
    gt_csv = "tests/sample_gt.csv"
    val_color_csv = "tests/sample_val_color.csv"
    
    input_df = pd.DataFrame({
        's3_image_path': ['data/pp_images/bacon_1103.jpg', 'data/pp_images/bacon_1303.jpg', 'data/pp_images/bacon_1503.jpg'],
        'site_id': ['S001', 'S001', 'S001'],
        'taxonomy_code': ['T001', 'T001', 'T001'],
        'input_groundtruth': [400, 120, 300]
    })
    input_df.to_csv(input_csv, index=False)

    gt_df = pd.DataFrame({
        'taxonomy_code': ['T001', 'T001'],
        'reference_pixel_count': [818671, 257877],
        'groundtruth': [400, 120]
    })
    gt_df.to_csv(gt_csv, index=False)

    val_color_df = pd.DataFrame({
        'taxonomy_code': ['T001'],
        'site_id': ['S001'],
        'colors': ['[(208,170,133),(154,123,100),(81,62,43),(236,208,168),(67,52,36)]']
    })
    val_color_df.to_csv(val_color_csv, index=False)

    output_csv_path, results = process_pipeline(input_csv, gt_csv, val_color_csv, logger)

    assert os.path.exists(output_csv_path)

    output_df = pd.read_csv(output_csv_path)

    expected_columns = ['s3_image_path', 'site_id', 'taxonomy_code', 'input_groundtruth', 'mask', 'final_image', 'pixel_count', 'histogram', 'pred_w2', 'error', 'success']
    assert all([col in output_df.columns for col in expected_columns])

    assert len(results) == 3 

   
    assert all(output_df['success'])


    os.remove(input_csv)
    os.remove(gt_csv)
    os.remove(val_color_csv)
    os.remove(output_csv_path)


if __name__ == "__main__":
    pytest.main()



import argparse


def main(args):
    input_csv = args.input_csv
    res = final_func(input_csv,**kwargs)
    pass

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Weight prediction")
    parser.add_argument("--input_csv", help="Input CSV file")
    parser.add_argument("--output_csv", help="Output CSV file")
    args = parser.parse_args()
    main(args)
    



### File for getting weight estimates from images

import transformers as t
import cv2
from PIL import Image
import numpy as np
import matplotlib.pyplot as plt
import tensorflow as tf
from rembg import remove ## for background removal -needed for some sites 
import skimage
from skimage.util import img_as_ubyte

__all__ = ["get_segformer", "get_biggest_contours", "get_seg", "get_final_mask", "get_final_img", "get_final_mask_filter",
           "get_histogram", "get_weight", "get_val", "split_filename"]

model_checkpoint = '/home/malav/wml_packges/mit-segformer-s' # download one of the pretrained models from S3 to your local device

def get_segformer(model_checkpoint=None) :
    """
    Function to get the Segformer model
    """
    return t.TFSegformerForSemanticSegmentation.from_pretrained(model_checkpoint)


def get_biggest_contours(contours):
    """
    Returns the biggest contour from a list of contours
    """
    val = 0
    final_contour = [contours[0]]
    for counter in contours:
        x,y,width,height = cv2.boundingRect(counter)
        #w_val = width-x
        #h_val = height-y
        #area = w_val*h_val
        area = width*height
        if area>val:
            val = area
            final_contour=[x,y,width,height]
    return final_contour


def get_seg(img,model,mask_val=0.08,resize=True,new_bg_removal=False,equalize=False,show=False,logger=None):
    """
    Function to get the segmentation mask
    """
    if isinstance(img,str):
        if new_bg_removal:
            image = Image.fromarray(np.array(remove(Image.open(img))))
        else:
            image = Image.open(img)
        if show:
            plt.imshow(image)
    else:
        if new_bg_removal:
            image = Image.fromarray(np.array(remove(img)))
        else:
            image = Image.open(img) 
        if show:
            plt.imshow(image)   
    if equalize:
        image = Image.fromarray(img_as_ubyte(skimage.exposure.equalize_adapthist(np.array(image), clip_limit=0.03)))
        if show:
            plt.imshow(image)
    if resize:
        image = image.resize((1600,1200))
    image_ori = image.copy()
    image_np = np.array(image_ori)
    
    mean = tf.constant([0.485, 0.456, 0.406])
    std = tf.constant([0.229, 0.224, 0.225])
    
    image = tf.convert_to_tensor(image_np)[:, :, :3]
    before_im = tf.convert_to_tensor(np.zeros(image.shape, dtype=np.float32))
    after_im = (tf.cast(image, tf.float32)/255. - mean) / std
    tensor = tf.concat([before_im,  after_im], axis=-1)
    tensor = tf.image.resize(tf.expand_dims(tensor, axis=0), [512, 512])
    tensor = tf.transpose(tensor, (0, 3, 1, 2))
    mask = model(tensor).logits[0, 1]
    mask = tf.image.resize(tf.expand_dims(tf.math.sigmoid(mask), -1), image.shape[:2]).numpy()
    mask_old = mask.copy()
    
    mask = np.where(mask>mask_val,mask,0)
    mask = np.where(mask<mask_val,mask,1)
    
    mask = np.array(mask, np.uint8)
    mask_res_np = np.array(mask)
    
    # contours, _ = cv2.findContours(mask_res_np[:,:], cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_TC89_KCOS)
    # for contour in contours:
    #     x, y, width, height = cv2.boundingRect(contour)
    #     print("Bounding Box Coordinates: ", x, y, x + width, y + height)
    # biggest_contours = get_biggest_contours(contours)
    # if logger:
    #     logger.info(biggest_contours)
    mask = cv2.normalize(mask_res_np, None, alpha=0, beta=1, norm_type=cv2.NORM_MINMAX, dtype=cv2.CV_32F)
    
    mask_box = np.zeros(mask_res_np.shape, dtype=np.uint8)
    #cv2.rectangle(mask_box, (biggest_contours[0], biggest_contours[1],biggest_contours[2], biggest_contours[3]), 255, -1)    
    mask_res_np[mask_box <= 0.5] = 0
    #plt.imshow(mask_box)

    masked_image = np.copy(image_np)
    masked_image[mask<= 0.5] = 0
    #plt.imshow(masked_image)
    
    if equalize==False:
        masked_image = masked_image[:,:,:3]
        
    if show:
        f1, axarr1 = plt.subplots(1,2,figsize=(15,15),gridspec_kw={'wspace':0.1, 'hspace':0.1},squeeze=True)        
        axarr1[0].imshow(mask_box,cmap='gray')
        axarr1[1].imshow(masked_image)
       
    # color = (0, 255, 0) 
    # thickness = 2
    # cv2.rectangle(image_np, (biggest_contours[0], biggest_contours[1], biggest_contours[2], biggest_contours[3]), color, thickness)

    if show:
        f, axarr = plt.subplots(2,2,figsize=(15,15),gridspec_kw={'wspace':0.1, 'hspace':0.1},squeeze=True)
        axarr[0,0].imshow(image)
        axarr[0,1].imshow(mask_old)
        axarr[1,0].imshow(mask)
        axarr[1,1].imshow(image_np)   
    
    return masked_image

def get_final_mask(masked_img,color,val =55,logger=None):
    """
    Function to get the final mask
    """
    upper_bound = [i+val for i in color]
    lower_bound = [i-val for i in color]
    # if logger:
    #     logger.info(upper_bound)
    #     logger.info(lower_bound)
    final_img = cv2.inRange(masked_img, np.array(lower_bound), np.array(upper_bound))

    # new_masked_image = masked_img.copy()
    # new_masked_image[final_img<= 0.5] = 0
    # if show:
    #     f, (axarr,axarr2) = plt.subplots(1,2,figsize=(15,15),gridspec_kw={'wspace':0.1, 'hspace':0.1},squeeze=True)
    #     axarr.imshow(final_img)
    #     axarr2.imshow(new_masked_image)
    return final_img

def get_final_mask_filter(masked_image,colors=None,val=55,show=False,logger=None):   
    # new_image = masked_image.copy()

    if colors is None:
        return masked_image
    
    # for color in colors:
    #     new_image = get_final_mask(new_image,color,val,show=show,logger=logger)
    # new_image[(new_image < 15).all(axis=2)] = [0, 0, 0]
    # return new_image
    combined_mask = np.zeros(masked_image.shape[:2], dtype=np.uint8)

    for color in colors:
        final_mask = get_final_mask(masked_image, color, val, logger=logger)
        combined_mask = cv2.bitwise_or(combined_mask, final_mask)
    
    # Apply the combined mask to the original image
    new_masked_image = masked_image.copy()
    new_masked_image[combined_mask <= 0.5] = 0
    
    if show:
        f, (axarr, axarr2) = plt.subplots(1, 2, figsize=(15, 15), gridspec_kw={'wspace': 0.1, 'hspace': 0.1}, squeeze=True)
        axarr.imshow(combined_mask, cmap='gray')
        axarr2.imshow(new_masked_image)
    
    return new_masked_image


def get_final_img(img,mask,mask_val=0.3):
    """
    Function to get the final image
    """
    new_masked_image = np.copy(Image.open(img))
    if new_masked_image.shape[0]==1600:
        new_masked_image = cv2.resize(new_masked_image, (1600, 1200), interpolation = cv2.INTER_AREA)
    new_masked_image[mask<= mask_val] = 0
    return new_masked_image
    
def get_histogram(img,show=False,logger=None):
    """
    Function to get the histogram of the image
    """
    img = np.where(img<10,img, 100)
    histogram, bin_edges, patches = plt.hist(img.ravel())
    if logger:
        logger.info('histogram :  {}'.format(histogram))
        logger.info('bin edges : {}'.format(bin_edges))
    if show:
        plt.figure()
        plt.plot(bin_edges[:-1], histogram)
        plt.show()
    return histogram, bin_edges

def get_weight(img,model,**kwargs):
    """
    Function to get the weight of the object in the image
    """
    mask = get_seg(img,model,**kwargs)
    get_histogram(mask,)
    return np.sum(mask)

def get_val(pix1,pix2,weight1,weight2,logger=None):
    """
    Function to get the histogram error
    """
    pred_w2 = weight1*(pix2/pix1)
    if weight2 == 0:
        error = 0.0
    else:
        error = ((abs(weight2-pred_w2))/weight2)*100 if weight2 !=0 else 0.0
    if logger:
        logger.info(pred_w2)
        logger.info(error)
    return pred_w2,error

# def get_reference_data(file_name,model,data_file_path,logger=None):
#     """
#     Function to get the reference data of each reference image with model version
#     """

#     pass

def split_filename(file: str,logger=None) -> dict :
    """
    Function to split the filename into parts
    """
    parts = file.split('/')
    if logger:
        logger.info(parts)
    return {
        'site_id': parts[0][7:],
        'meal_service': parts[1][12:],
        'food_item': parts[2][9:],
        'image': parts[3][6:-13]
    }


## File for S3 I/O operations (uploading, downloading, etc.)

import boto3
from io import BytesIO
from PIL import Image
import pandas as pd
from mb_pandas.src import remove_unnamed
import io


__all__ = ["upload_df", "download_df", "upload_image", "download_image","get_response","get_image"]

def get_client():
    """
    Returns a boto3 client for S3
    """
    return boto3.client('s3')

def get_response(client, bucket, key):
    """
    Returns the response object for a given bucket and key
    """
    return client.get_object(Bucket=bucket, Key=key)

def get_image(response):
    """
    Returns an image object from the response object
    """
    return Image.open(BytesIO(response['Body'].read()))

def upload_df(df, bucket, key, client=None):
    """
    Uploads a dataframe to S3
    """
    if client is None:
        client = get_client()
    df = remove_unnamed(df)
    with io.StringIO() as csv_buffer:
        df.to_csv(csv_buffer, index=False)
    
        response = client.put_object(
            Bucket=bucket, Key=key, Body=csv_buffer.getvalue())

        status = response.get("ResponseMetadata", {}).get("HTTPStatusCode") 
        if status == 200:
            return True
        else:
            return False
        
def download_df(bucket, key, client=None):
    """ 
    Downloads a dataframe from S3
    """
    if client is None:
        client = get_client()
    response = get_response(client, bucket, key)
    return pd.read_csv(response['Body'])

def upload_image(image_path, bucket, key, client=None):
    """
    Uploads an image/object to S3
    """
    if client is None:
        client = get_client()
    #client.put_object(Bucket=bucket, Key=key, Body=image)
    client.upload_file(image_path,Bucket=bucket,Key=key)

def download_image(bucket, key, client=None):
    """
    Downloads an image from S3
    """
    if client is None:
        client = get_client()
    response = get_response(client, bucket, key)
    return get_image(response)
    


## function to send csv file to slack channel

from slack_sdk import WebhookClient, WebClient
import tabulate

__all__ = ['send_file_to_slack','send_message_to_slack','get_slack_webhook']

def send_file_to_slack(file_path,slack_token,slack_channel):
    """
    Function to send the file to slack channel
    """
    try :
        
        client = WebClient(token=slack_token)
        response = client.files_upload(
            channels=slack_channel,
            file=file_path)
        return response
    except Exception as e:
        return e
    
# slack_webhook = 'https://hooks.slack.com/services/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' ## pp_image_weight_estimation

def get_slack_webhook(slack_channel='pp_image_weight_estimation'):
    """
    Function to get the slack webhook
    """
    if slack_channel=='pp_image_weight_estimation':
        slack_webhook = "https://hooks.slack.com/services/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"
        #slack_webhook = 'https://hooks.slack.com/services/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'
        #slack_webhook = 'https://hooks.slack.com/services/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'
        #slack_webhook = 'https://hooks.slack.com/services/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'
    else:
        return 'Invalid slack channel'
    return slack_webhook

## new slack_webhook ='https://hooks.slack.com/services/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'

def send_message_to_slack(message,slack_channel='pp_image_weight_estimation',is_df=False,tabulate_type='fancy_grid',url=None,**kwargs):
    """
    Function to send the message to slack channel. Msg should be a json file. (CSV file not good for this function)
    Args:
        message : str : message to be sent or pd.DataFrame : dataframe to be sent
        is_df : bool : whether the message is a dataframe or not. If True, the message will be taken as a string to be loaded in a dataframe
        slack_webhook : str : webhook url for the slack channel
        tabulate_type : str : type of tabulation to be done. Default is 'orgtbl'. Other is 'grid'
    """
    try :
        slack_webhook = get_slack_webhook(slack_channel)
        if slack_webhook == 'Invalid slack channel':
            slack_webhook = url
        if url:
            slack_webhook = url
        client = WebhookClient(url=slack_webhook)
        if is_df:
            #df = pd.read_csv(message)
            tab = (tabulate.tabulate(message,headers=list(message.columns), tablefmt=tabulate_type,floatfmt='.3f',**kwargs))
            slack_table = {"text":"```\n" + tab + "\n```"}
            info = slack_table['text']
            response = client.send(text=info)
        else:

            response = client.send(text=message)
        return response
    except Exception as e:
        return e